searchData={"items":[{"type":"module","title":"Eliot","doc":"Eliot - Production-ready IoT data ingestion system.\n\nProvides fault-tolerant MQTT communication, structured logging,\nand comprehensive error handling for scalable device fleet management.","ref":"Eliot.html"},{"type":"module","title":"Features - Eliot","doc":"- **Fault-tolerant MQTT communication** using Tortoise client\n- **Structured logging** with telemetry integration\n- **Comprehensive error handling** with retry mechanisms\n- **Production-ready** with OTP supervision trees\n- **Scalable architecture** for device fleet management","ref":"Eliot.html#module-features"},{"type":"module","title":"Quick Start - Eliot","doc":"# Start the application\n    {:ok, _} = Application.start(:eliot)\n    \n    # The MQTT client and supervisors start automatically","ref":"Eliot.html#module-quick-start"},{"type":"module","title":"Configuration - Eliot","doc":"Configure MQTT broker settings in your application config:\n\n    config :eliot,\n      mqtt_broker: \"localhost\",\n      mqtt_port: 1883,\n      client_id: \"eliot_client\"","ref":"Eliot.html#module-configuration"},{"type":"module","title":"Architecture - Eliot","doc":"Eliot uses OTP supervision trees to ensure fault tolerance:\n\n- `Eliot.Application` - Main application supervisor\n- `Eliot.Logger` - Structured logging with telemetry\n- `Eliot.ErrorHandler` - Centralized error handling and recovery","ref":"Eliot.html#module-architecture"},{"type":"function","title":"Eliot.config/2","doc":"Returns configuration for the given key.","ref":"Eliot.html#config/2"},{"type":"function","title":"Eliot.start_mqtt_connection/0","doc":"Starts MQTT connection with the configured broker.","ref":"Eliot.html#start_mqtt_connection/0"},{"type":"function","title":"Eliot.version/0","doc":"Returns the version of Eliot.","ref":"Eliot.html#version/0"},{"type":"module","title":"Eliot.Application","doc":"The Eliot Application.\n\nStarts and supervises all components of the IoT data ingestion system,\nincluding MQTT connections, logging infrastructure, and error handling.","ref":"Eliot.Application.html"},{"type":"function","title":"Eliot.Application.health_check/0","doc":"Returns the current status of all supervised children.","ref":"Eliot.Application.html#health_check/0"},{"type":"function","title":"Eliot.Application.supervisor_info/0","doc":"Returns the application supervision tree information.","ref":"Eliot.Application.html#supervisor_info/0"},{"type":"module","title":"Eliot.ErrorHandler","doc":"Centralized error handling and recovery for the Eliot IoT system.\n\nProvides retry mechanisms, circuit breaker patterns, and comprehensive\nerror tracking for resilient device communication and data processing.","ref":"Eliot.ErrorHandler.html"},{"type":"function","title":"Eliot.ErrorHandler.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Eliot.ErrorHandler.html#child_spec/1"},{"type":"function","title":"Eliot.ErrorHandler.circuit_open?/0","doc":"Checks if the circuit breaker is open (blocking requests).","ref":"Eliot.ErrorHandler.html#circuit_open?/0"},{"type":"function","title":"Eliot.ErrorHandler.get_stats/0","doc":"Returns error handling statistics.","ref":"Eliot.ErrorHandler.html#get_stats/0"},{"type":"function","title":"Eliot.ErrorHandler.handle_error/3","doc":"Handles an error with automatic retry and circuit breaker logic.","ref":"Eliot.ErrorHandler.html#handle_error/3"},{"type":"function","title":"Eliot.ErrorHandler.reset_circuit/0","doc":"Resets the circuit breaker to closed state.","ref":"Eliot.ErrorHandler.html#reset_circuit/0"},{"type":"function","title":"Eliot.ErrorHandler.start_link/1","doc":"Starts the error handler GenServer.","ref":"Eliot.ErrorHandler.html#start_link/1"},{"type":"function","title":"Eliot.ErrorHandler.with_retry/3","doc":"Executes a function with automatic error handling and retries.","ref":"Eliot.ErrorHandler.html#with_retry/3"},{"type":"module","title":"Eliot.Logger","doc":"Centralized logging for the Eliot IoT data ingestion system.\n\nProvides structured logging with telemetry integration for comprehensive\nobservability of device communications, data processing, and system events.","ref":"Eliot.Logger.html"},{"type":"module","title":"Features - Eliot.Logger","doc":"- Structured logging with metadata enrichment\n- Telemetry event integration for system monitoring\n- Device-specific event tracking\n- MQTT connection lifecycle logging\n- Data processing performance metrics\n- Automatic statistics reporting","ref":"Eliot.Logger.html#module-features"},{"type":"module","title":"Usage - Eliot.Logger","doc":"# Basic logging\n    Eliot.Logger.log_info(\"System started\", %{component: \"application\"})\n    Eliot.Logger.log_error(\"Connection failed\", %{retry_count: 3})\n\n    # Device event logging\n    Eliot.Logger.log_device_event(\"sensor_001\", \"temperature_reading\", %{\n      temperature: 23.5,\n      humidity: 65.0\n    })\n\n    # MQTT event logging\n    broker_config = %{host: \"mqtt.example.com\", port: 1883}\n    Eliot.Logger.log_mqtt_event(\"connection_established\", broker_config)\n\n    # Processing event logging\n    Eliot.Logger.log_processing_event(\"msg_123\", 150, \"success\")\n\nAll logging functions automatically emit corresponding telemetry events\nfor integration with monitoring and alerting systems.","ref":"Eliot.Logger.html#module-usage"},{"type":"function","title":"Eliot.Logger.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Eliot.Logger.html#child_spec/1"},{"type":"function","title":"Eliot.Logger.handle_telemetry_event/4","doc":"Handles telemetry events for system-wide logging.\n\nThis function is defined as a module function to avoid the telemetry\nperformance warning about local functions.","ref":"Eliot.Logger.html#handle_telemetry_event/4"},{"type":"function","title":"Eliot.Logger.log_device_event/3","doc":"Logs device-specific events with standardized metadata.\n\nAutomatically enriches the event with timestamp and device information,\nthen emits a telemetry event for monitoring systems.","ref":"Eliot.Logger.html#log_device_event/3"},{"type":"function","title":"Parameters - Eliot.Logger.log_device_event/3","doc":"- `device_id` - Unique identifier for the device\n- `event_type` - Type of event (e.g., \"sensor_reading\", \"connection_lost\")\n- `data` - Optional event-specific data (default: `%{}`)","ref":"Eliot.Logger.html#log_device_event/3-parameters"},{"type":"function","title":"Examples - Eliot.Logger.log_device_event/3","doc":"# Simple device event\n    Eliot.Logger.log_device_event(\"sensor_001\", \"online\")\n\n    # Device event with sensor data\n    Eliot.Logger.log_device_event(\"thermometer_42\", \"temperature_reading\", %{\n      temperature: 23.5,\n      humidity: 65.0,\n      battery_level: 87\n    })","ref":"Eliot.Logger.html#log_device_event/3-examples"},{"type":"function","title":"Eliot.Logger.log_error/2","doc":"Logs an error-level message with structured metadata.","ref":"Eliot.Logger.html#log_error/2"},{"type":"function","title":"Parameters - Eliot.Logger.log_error/2","doc":"- `message` - The log message string\n- `metadata` - Optional map of structured metadata (default: `%{}`)","ref":"Eliot.Logger.html#log_error/2-parameters"},{"type":"function","title":"Examples - Eliot.Logger.log_error/2","doc":"Eliot.Logger.log_error(\"Database connection failed\")\n    Eliot.Logger.log_error(\"API request timeout\", %{endpoint: \"/api/data\", timeout_ms: 5000})","ref":"Eliot.Logger.html#log_error/2-examples"},{"type":"function","title":"Eliot.Logger.log_info/2","doc":"Logs an info-level message with structured metadata.","ref":"Eliot.Logger.html#log_info/2"},{"type":"function","title":"Parameters - Eliot.Logger.log_info/2","doc":"- `message` - The log message string\n- `metadata` - Optional map of structured metadata (default: `%{}`)","ref":"Eliot.Logger.html#log_info/2-parameters"},{"type":"function","title":"Examples - Eliot.Logger.log_info/2","doc":"Eliot.Logger.log_info(\"System initialized successfully\")\n    Eliot.Logger.log_info(\"User authenticated\", %{user_id: \"123\", method: \"oauth\"})","ref":"Eliot.Logger.html#log_info/2-examples"},{"type":"function","title":"Eliot.Logger.log_mqtt_event/3","doc":"Logs MQTT-related events with connection metadata.\n\nTracks MQTT broker interactions including connections, disconnections,\nmessage publishing, and subscription events.","ref":"Eliot.Logger.html#log_mqtt_event/3"},{"type":"function","title":"Parameters - Eliot.Logger.log_mqtt_event/3","doc":"- `event_type` - Type of MQTT event (e.g., \"connection_established\", \"message_published\")\n- `broker_info` - Map containing broker connection details\n- `data` - Optional event-specific data (default: `%{}`)","ref":"Eliot.Logger.html#log_mqtt_event/3-parameters"},{"type":"function","title":"Examples - Eliot.Logger.log_mqtt_event/3","doc":"broker_config = %{host: \"mqtt.example.com\", port: 1883, client_id: \"eliot_001\"}\n    \n    # Connection event\n    Eliot.Logger.log_mqtt_event(\"connection_established\", broker_config)\n\n    # Message publishing event\n    Eliot.Logger.log_mqtt_event(\"message_published\", broker_config, %{\n      topic: \"sensors/temperature\",\n      qos: 1,\n      payload_size: 128\n    })","ref":"Eliot.Logger.html#log_mqtt_event/3-examples"},{"type":"function","title":"Eliot.Logger.log_processing_event/3","doc":"Logs data processing events with performance metrics.\n\nTracks message processing performance and outcomes for monitoring\nsystem throughput and identifying bottlenecks.","ref":"Eliot.Logger.html#log_processing_event/3"},{"type":"function","title":"Parameters - Eliot.Logger.log_processing_event/3","doc":"- `message_id` - Unique identifier for the processed message\n- `processing_time_ms` - Processing duration in milliseconds\n- `result` - Processing outcome (e.g., \"success\", \"error\", \"timeout\")","ref":"Eliot.Logger.html#log_processing_event/3-parameters"},{"type":"function","title":"Examples - Eliot.Logger.log_processing_event/3","doc":"# Successful processing\n    Eliot.Logger.log_processing_event(\"msg_12345\", 150, \"success\")\n\n    # Failed processing\n    Eliot.Logger.log_processing_event(\"msg_12346\", 2500, \"timeout\")","ref":"Eliot.Logger.html#log_processing_event/3-examples"},{"type":"function","title":"Eliot.Logger.log_warning/2","doc":"Logs a warning-level message with structured metadata.","ref":"Eliot.Logger.html#log_warning/2"},{"type":"function","title":"Parameters - Eliot.Logger.log_warning/2","doc":"- `message` - The log message string\n- `metadata` - Optional map of structured metadata (default: `%{}`)","ref":"Eliot.Logger.html#log_warning/2-parameters"},{"type":"function","title":"Examples - Eliot.Logger.log_warning/2","doc":"Eliot.Logger.log_warning(\"High memory usage detected\")\n    Eliot.Logger.log_warning(\"Rate limit approaching\", %{current_rate: 95, limit: 100})","ref":"Eliot.Logger.html#log_warning/2-examples"},{"type":"function","title":"Eliot.Logger.start_link/1","doc":"Starts the logger GenServer.","ref":"Eliot.Logger.html#start_link/1"},{"type":"function","title":"Options - Eliot.Logger.start_link/1","doc":"- `:name` - The name to register the GenServer under (default: `__MODULE__`)","ref":"Eliot.Logger.html#start_link/1-options"},{"type":"function","title":"Examples - Eliot.Logger.start_link/1","doc":"{:ok, pid} = Eliot.Logger.start_link()\n    {:ok, pid} = Eliot.Logger.start_link(name: MyLogger)","ref":"Eliot.Logger.html#start_link/1-examples"},{"type":"module","title":"Eliot.MessageParser","doc":"Handles decoding of standardized JSON messages from devices.","ref":"Eliot.MessageParser.html"},{"type":"function","title":"Eliot.MessageParser.parse/1","doc":"Parses a JSON-encoded message string into an Elixir map.\n\nReturns `{:ok, map}` on success or `{:error, reason}` if parsing fails.\nFailed parses are automatically logged as a warning.","ref":"Eliot.MessageParser.html#parse/1"},{"type":"function","title":"Examples - Eliot.MessageParser.parse/1","doc":"iex> valid_json = ~s({\n    ...>   \"device_id\": \"robot_001\",\n    ...>   \"timestamp\": \"2025-06-05T14:30:00Z\",\n    ...>   \"sensor_type\": \"gps\",\n    ...>   \"data\": { \"latitude\": 37.7749, \"longitude\": -122.4194 }\n    ...> })\n    iex> Eliot.MessageParser.parse(valid_json)\n    {:ok, %{\n      \"data\" => %{\"latitude\" => 37.7749, \"longitude\" => -122.4194},\n      \"device_id\" => \"robot_001\",\n      \"sensor_type\" => \"gps\",\n      \"timestamp\" => \"2025-06-05T14:30:00Z\"\n    }}\n\n    iex> invalid_json = ~s({\"device_id\": \"robot_001\",})\n    iex> match?({:error, %Jason.DecodeError{}}, Eliot.MessageParser.parse(invalid_json))\n    true","ref":"Eliot.MessageParser.html#parse/1-examples"},{"type":"extras","title":"Eliot - IoT Data Ingestion System","doc":"# Eliot - IoT Data Ingestion System\n\n> *\"Hello, friend. Welcome to the world of connected devices.\"*\n\nEliot is a production-ready IoT data ingestion system built with Elixir/OTP, designed for high-throughput sensor data processing and real-time fleet management. Born from the need to handle massive scale device communication with fault-tolerance and observability at its core.\n\n## 🤖 Features\n\n- **Real-time MQTT Communication** - Handle thousands of concurrent device connections\n- **Fault-Tolerant Architecture** - Supervisor trees that never sleep, built to survive any crash\n- **Structured Logging** - Every event tracked, every anomaly detected\n- **Circuit Breaker Patterns** - Automatic recovery from network failures\n- **Environment-Specific Configuration** - Seamless deployment from development to production\n- **Comprehensive Test Suite** - High coverage with integration tests covering error scenarios and edge cases\n- **Clean Code Quality** - Idiomatic Elixir following industry best practices\n\n## 🏗️ Architecture\n\nEliot follows the **\"let it crash\"** philosophy - embrace failure, isolate it, and recover gracefully. The system is built around an OTP application with isolated supervision trees for maximum resilience.\n\n```\n┌─────────────────────────────────────────────────────────────┐\n│                    Eliot Supervisor                         │\n├─────────────────────────────────────────────────────────────┤\n│  ┌─────────────────┐  ┌─────────────────┐  ┌──────────────┐ │\n│  │ Error Handler   │  │ Message Parser  │  │ Logger       │ │\n│  │ (Retry Logic)   │  │ (JSON Processing)│  │ (Telemetry)  │ │\n│  └─────────────────┘  └─────────────────┘  └──────────────┘ │\n└─────────────────────────────────────────────────────────────┘\n```\n\nThe architecture demonstrates understanding of:\n- **Distributed Systems** - Handle network partitions and node failures\n- **Event-Driven Design** - React to device events in real-time  \n- **Observability** - Full system visibility through structured logging\n- **Scalability** - Process millions of messages without breaking a sweat\n\n## 🚀 Quick Start","ref":"readme.html"},{"type":"extras","title":"Prerequisites - Eliot - IoT Data Ingestion System","doc":"- **Elixir 1.14+** - [Install Elixir](https://elixir-lang.org/install.html)\n- **Erlang/OTP 25+** - [Erlang Downloads](https://www.erlang.org/downloads)\n- **Git** - [Install Git](https://git-scm.com/downloads)\n- **MQTT Broker** (optional for development) - [Eclipse Mosquitto](https://mosquitto.org/)","ref":"readme.html#prerequisites"},{"type":"extras","title":"Installation - Eliot - IoT Data Ingestion System","doc":"```bash\n# Clone the repository\ngit clone https://github.com/christimahu/eliot.git\ncd eliot\n\n# Install dependencies\nmix deps.get\n\n# Run tests to verify everything works\nmix test\n\n# Start the application\nmix run --no-halt\n```","ref":"readme.html#installation"},{"type":"extras","title":"Development - Eliot - IoT Data Ingestion System","doc":"```bash\n# Run with interactive shell\niex -S mix\n\n# Run linting (if configured)\nmix credo\n\n# Format code\nmix format\n\n# Build production release\nMIX_ENV=prod mix release\n```","ref":"readme.html#development"},{"type":"extras","title":"Learning Resources - Eliot - IoT Data Ingestion System","doc":"- **[Elixir Official Guide](https://elixir-lang.org/getting-started/introduction.html)** - Learn Elixir fundamentals\n- **[OTP Design Principles](https://www.erlang.org/doc/design_principles/users_guide.html)** - Understanding supervision trees\n- **[MQTT Protocol](https://mqtt.org/)** - IoT messaging protocol documentation\n- **[Phoenix Framework](https://phoenixframework.org/)** - If adding web interfaces\n- **[Nerves Project](https://nerves-project.org/)** - For embedded IoT devices\n\n## 🔧 Configuration\n\nEliot supports environment-specific configuration:\n\n```elixir\n# config/prod.exs\nconfig :eliot,\n  mqtt: [\n    broker_host: System.get_env(\"MQTT_BROKER_HOST\"),\n    broker_port: 8883,\n    ssl: true,\n    keepalive: 300\n  ]\n```","ref":"readme.html#learning-resources"},{"type":"extras","title":"Environment Variables - Eliot - IoT Data Ingestion System","doc":"- `MQTT_BROKER_HOST` - Production MQTT broker hostname\n- `MQTT_USERNAME` - Authentication username  \n- `MQTT_PASSWORD` - Authentication password\n- `MQTT_CLIENT_CERT_FILE` - Client certificate for mutual TLS\n\n## 📊 Message Format\n\nAll device messages follow a standardized JSON schema:\n\n```json\n{\n  \"device_id\": \"robot_001\",\n  \"timestamp\": \"2025-06-05T14:30:00Z\", \n  \"sensor_type\": \"gps\",\n  \"data\": {\n    \"latitude\": 37.7749,\n    \"longitude\": -122.4194,\n    \"accuracy\": 3.2\n  }\n}\n```\n\n## 🛡️ Error Handling\n\nEliot implements sophisticated error handling patterns:\n\n- **Exponential Backoff** - Automatic retry with increasing delays\n- **Circuit Breakers** - Fail fast when services are down\n- **Dead Letter Queues** - Preserve unprocessable messages for analysis\n- **Graceful Degradation** - Continue operating with reduced functionality\n\n```elixir\n# Example: Device connection timeout handling\n{:retry, %{device_id: \"robot_001\", backoff_ms: 2000}}\n\n# Example: Authentication failure handling  \n{:circuit_break, %{device_id: \"robot_002\", reason: :auth_failure}}\n```\n\n## 📈 Monitoring & Observability\n\nEvery operation generates structured logs for production monitoring:\n\n```elixir\n# Device connection events\nEliot.Logger.log_device_event(\"robot_001\", :connected)\n\n# Message processing metrics\nEliot.Logger.log_processing_event(\"message_123\", 150, :ok)\n\n# Security events\nEliot.Logger.log_error(\"Authentication failure\", %{device_id: \"unknown\", reason: :auth_failure})\n```\n\n## 🧪 Testing\n\n```bash\n# Run all tests\nmix test\n\n# Run with coverage\nmix test --cover\n\n# Run integration tests only\nmix test test/eliot/integration/\n\n# Run tests excluding integration tests\nmix test --exclude integration\n```\n\nThe test suite covers:\n- **Unit Tests** - Individual module functionality\n- **Integration Tests** - MQTT connection lifecycle\n- **Error Scenarios** - Network failures and malformed data\n- **Configuration Validation** - Environment-specific settings\n\nSee [TESTING.md](TESTING.md) for detailed testing documentation.\n\n## 🏭 Production Deployment","ref":"readme.html#environment-variables"},{"type":"extras","title":"Building a Release - Eliot - IoT Data Ingestion System","doc":"```bash\n# Create production release\nMIX_ENV=prod mix release\n\n# Run the release\n_build/prod/rel/eliot/bin/eliot start\n```","ref":"readme.html#building-a-release"},{"type":"extras","title":"Docker Deployment - Eliot - IoT Data Ingestion System","doc":"```dockerfile\nFROM elixir:1.14-alpine\nWORKDIR /app\nCOPY mix.exs mix.lock ./\nRUN mix deps.get --only prod\nCOPY . .\nRUN MIX_ENV=prod mix release\nCMD [\"_build/prod/rel/eliot/bin/eliot\", \"start\"]\n```\n\n## 🤝 Contributing\n\nWe welcome contributions! Please read our [Contributing Guide](CONTRIBUTING.md) and [Code of Conduct](CODE_OF_CONDUCT.md):\n\n1. Fork the repository\n2. Create a feature branch: `git checkout -b feature/new-capability`\n3. Write tests for your changes\n4. Ensure all tests pass: `mix test`\n5. Run linting: `mix credo` (if configured)\n6. Submit a pull request\n\n## 📋 Requirements Checklist\n\n- [x] Production-ready error handling\n- [x] Comprehensive test coverage with integration tests\n- [x] Environment-specific configuration\n- [x] Structured logging for observability\n- [x] Circuit breaker patterns\n- [x] MQTT protocol support\n- [x] Clean code quality\n- [x] Documentation with examples\n- [x] Graceful shutdown handling\n\n## 🏆 Quality Metrics\n\n```\nTests:     Comprehensive unit and integration test coverage\nCoverage:  High coverage of critical paths\nCredo:     Clean code following Elixir best practices\nFormat:    All code properly formatted\nDocs:      Comprehensive with examples\n```\n\n## 🔮 Roadmap\n\n- **Phase 1**: Core MQTT workers and message processing (Current)\n- **Phase 2**: HTTP API for fleet management\n- **Phase 3**: Real-time dashboards and alerting\n- **Phase 4**: Machine learning anomaly detection\n- **Phase 5**: Multi-region deployment and edge computing\n\n## 📚 Documentation\n\n- **Testing**: See [TESTING.md](TESTING.md) for detailed testing information\n- **Changes**: See [CHANGELOG.md](CHANGELOG.md) for version history\n- **Contributing**: See [CONTRIBUTING.md](CONTRIBUTING.md) for contribution guidelines\n\n## 📄 License\n\nThis project is licensed under the GNU General Public License v3.0 - see the [LICENSE](LICENSE) file for details.\n\n---\n\n*\"The world is a dangerous place, not because of those who do evil, but because of those who look on and do nothing. Eliot watches, processes, and acts.\"*\n\nBuilt with ❤️ and lots of ☕ for the IoT revolution.","ref":"readme.html#docker-deployment"},{"type":"extras","title":"Contributor Covenant Code of Conduct","doc":"# Contributor Covenant Code of Conduct","ref":"code_of_conduct.html"},{"type":"extras","title":"Our Pledge - Contributor Covenant Code of Conduct","doc":"We as members, contributors, and leaders pledge to make participation in our\ncommunity a harassment-free experience for everyone, regardless of age, body\nsize, visible or invisible disability, ethnicity, sex characteristics, gender\nidentity and expression, level of experience, education, socio-economic status,\nnationality, personal appearance, race, caste, color, religion, or sexual\nidentity and orientation.\n\nWe pledge to act and interact in ways that contribute to an open, welcoming,\ndiverse, inclusive, and healthy community.","ref":"code_of_conduct.html#our-pledge"},{"type":"extras","title":"Our Standards - Contributor Covenant Code of Conduct","doc":"Examples of behavior that contributes to a positive environment for our\ncommunity include:\n\n* Demonstrating empathy and kindness toward other people\n* Being respectful of differing opinions, viewpoints, and experiences\n* Giving and gracefully accepting constructive feedback\n* Accepting responsibility and apologizing to those affected by our mistakes,\n  and learning from the experience\n* Focusing on what is best not just for us as individuals, but for the overall\n  community\n\nExamples of unacceptable behavior include:\n\n* The use of sexualized language or imagery, and sexual attention or advances of\n  any kind\n* Trolling, insulting or derogatory comments, and personal or political attacks\n* Public or private harassment\n* Publishing others' private information, such as a physical or email address,\n  without their explicit permission\n* Other conduct which could reasonably be considered inappropriate in a\n  professional setting","ref":"code_of_conduct.html#our-standards"},{"type":"extras","title":"Enforcement Responsibilities - Contributor Covenant Code of Conduct","doc":"Community leaders are responsible for clarifying and enforcing our standards of\nacceptable behavior and will take appropriate and fair corrective action in\nresponse to any behavior that they deem inappropriate, threatening, offensive,\nor harmful.\n\nCommunity leaders have the right and responsibility to remove, edit, or reject\ncomments, commits, code, wiki edits, issues, and other contributions that are\nnot aligned to this Code of Conduct, and will communicate reasons for moderation\ndecisions when appropriate.","ref":"code_of_conduct.html#enforcement-responsibilities"},{"type":"extras","title":"Scope - Contributor Covenant Code of Conduct","doc":"This Code of Conduct applies within all community spaces, and also applies when\nan individual is officially representing the community in public spaces.\nExamples of representing our community include using an official email address,\nposting via an official social media account, or acting as an appointed\nrepresentative at an online or offline event.","ref":"code_of_conduct.html#scope"},{"type":"extras","title":"Enforcement - Contributor Covenant Code of Conduct","doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be\nreported to the community leaders responsible for enforcement at\n[https://christimahu.com/contact](https://christimahu.com/contact).\nAll complaints will be reviewed and investigated promptly and fairly.\n\nAll community leaders are obligated to respect the privacy and security of the\nreporter of any incident.","ref":"code_of_conduct.html#enforcement"},{"type":"extras","title":"Enforcement Guidelines - Contributor Covenant Code of Conduct","doc":"Community leaders will follow these Community Impact Guidelines in determining\nthe consequences for any action they deem in violation of this Code of Conduct:","ref":"code_of_conduct.html#enforcement-guidelines"},{"type":"extras","title":"1. Correction - Contributor Covenant Code of Conduct","doc":"**Community Impact**: Use of inappropriate language or other behavior deemed\nunprofessional or unwelcome in the community.\n\n**Consequence**: A private, written warning from community leaders, providing\nclarity around the nature of the violation and an explanation of why the\nbehavior was inappropriate. A public apology may be requested.","ref":"code_of_conduct.html#1-correction"},{"type":"extras","title":"2. Warning - Contributor Covenant Code of Conduct","doc":"**Community Impact**: A violation through a single incident or series of\nactions.\n\n**Consequence**: A warning with consequences for continued behavior. No\ninteraction with the people involved, including unsolicited interaction with\nthose enforcing the Code of Conduct, for a specified period of time. This\nincludes avoiding interactions in community spaces as well as external channels\nlike social media. Violating these terms may lead to a temporary or permanent\nban.","ref":"code_of_conduct.html#2-warning"},{"type":"extras","title":"3. Temporary Ban - Contributor Covenant Code of Conduct","doc":"**Community Impact**: A serious violation of community standards, including\nsustained inappropriate behavior.\n\n**Consequence**: A temporary ban from any sort of interaction or public\ncommunication with the community for a specified period of time. No public or\nprivate interaction with the people involved, including unsolicited interaction\nwith those enforcing the Code of Conduct, is allowed during this period.\nViolating these terms may lead to a permanent ban.","ref":"code_of_conduct.html#3-temporary-ban"},{"type":"extras","title":"4. Permanent Ban - Contributor Covenant Code of Conduct","doc":"**Community Impact**: Demonstrating a pattern of violation of community\nstandards, including sustained inappropriate behavior, harassment of an\nindividual, or aggression toward or disparagement of classes of individuals.\n\n**Consequence**: A permanent ban from any sort of public interaction within the\ncommunity.","ref":"code_of_conduct.html#4-permanent-ban"},{"type":"extras","title":"Attribution - Contributor Covenant Code of Conduct","doc":"This Code of Conduct is adapted from the [Contributor Covenant][homepage],\nversion 2.1, available at\n[https://www.contributor-covenant.org/version/2/1/code_of_conduct.html][v2.1].\n\nCommunity Impact Guidelines were inspired by\n[Mozilla's code of conduct enforcement ladder][Mozilla CoC].\n\nFor answers to common questions about this code of conduct, see the FAQ at\n[https://www.contributor-covenant.org/faq][FAQ]. Translations are available at\n[https://www.contributor-covenant.org/translations][translations].\n\n[homepage]: https://www.contributor-covenant.org\n[v2.1]: https://www.contributor-covenant.org/version/2/1/code_of_conduct.html\n[Mozilla CoC]: https://github.com/mozilla/inclusion\n[FAQ]: https://www.contributor-covenant.org/faq\n[translations]: https://www.contributor-covenant.org/translations","ref":"code_of_conduct.html#attribution"},{"type":"extras","title":"Contributing to Eliot","doc":"# Contributing to Eliot\n\nThank you for considering contributing to Eliot! This document outlines the process for contributing to the project and helps to ensure your contributions can be efficiently reviewed and potentially integrated into the codebase.","ref":"contributing.html"},{"type":"extras","title":"Code of Conduct - Contributing to Eliot","doc":"This project adheres to the [Contributor Covenant](CODE_OF_CONDUCT.md). By participating, you are expected to uphold this code. Please report unacceptable behavior via [https://christimahu.com/contact](https://christimahu.com/contact).","ref":"contributing.html#code-of-conduct"},{"type":"extras","title":"How Can I Contribute? - Contributing to Eliot","doc":"","ref":"contributing.html#how-can-i-contribute"},{"type":"extras","title":"Reporting Bugs - Contributing to Eliot","doc":"Bug reports are tracked as [GitHub issues](https://github.com/christimahu/eliot/issues). When creating a bug report, please include as much detail as possible:\n\n1. **Use a clear and descriptive title** for the issue to identify the problem.\n2. **Describe the exact steps to reproduce the problem** in as much detail as possible.\n3. **Provide specific examples** or code snippets to demonstrate the steps.\n4. **Describe the behavior you observed** after following the steps and why you consider it a bug.\n5. **Explain the behavior you expected** to see instead and why.\n6. **Include details about your environment** (Elixir version, Erlang/OTP version, OS).","ref":"contributing.html#reporting-bugs"},{"type":"extras","title":"Suggesting Enhancements - Contributing to Eliot","doc":"Enhancement suggestions are also tracked as [GitHub issues](https://github.com/christimahu/eliot/issues). When suggesting an enhancement:\n\n1. **Use a clear and descriptive title** for the issue.\n2. **Provide a detailed description of the suggested enhancement** and its expected behavior.\n3. **Explain why this enhancement would be useful** to IoT developers and the broader Elixir community.\n4. **List similar libraries or tools where this enhancement exists**, if applicable.","ref":"contributing.html#suggesting-enhancements"},{"type":"extras","title":"Code Contributions - Contributing to Eliot","doc":"#### Development Workflow\n\n1. **Fork the repository** on GitHub.\n2. **Clone your fork** to your local machine.\n3. **Create a new branch** from the `main` branch for your changes (`git checkout -b feature/my-new-feature`).\n4. **Make your changes** following the coding conventions described below.\n5. **Write or adapt tests** as necessary.\n6. **Ensure all tests pass** by running `mix test`.\n7. **Ensure code is formatted** by running `mix format`.\n8. **Check code quality** with `mix credo`.\n9. **Commit your changes** with clear commit messages.\n10. **Push your branch** to your fork on GitHub.\n11. **Submit a Pull Request** from your branch to the project's `main` branch.\n\n#### Pull Request Process\n\n1. Ensure your code follows the coding conventions.\n2. Update documentation as necessary.\n3. Add tests that verify your changes.\n4. Make sure all tests pass and that test coverage is maintained or improved.\n5. The PR should clearly describe the problem and solution.\n6. The PR will be reviewed by at least one maintainer.\n\n#### Coding Conventions\n\n- **Code Style**: The codebase follows the official [Elixir Style Guide](https://github.com/elixir-lang/elixir/blob/main/lib/elixir/pages/Style%20Guide.md). We use [Credo](https://github.com/rrrene/credo) for static analysis to enforce these standards.\n- **Formatting**: We use `mix format` for automated code formatting. Please run it before committing your changes.\n- **Naming Conventions**:\n  - Use `snake_case` for variables and function names.\n  - Use `CamelCase` for module names.\n- **Comments and Documentation**:\n  - Use ExDoc-style comments for all public modules and functions (`@moduledoc` and `@doc`).\n  - Write clear comments to explain complex or non-obvious code.\n  - Keep comments up-to-date with code changes.\n- **Error Handling**:\n  - Use `{:ok, value}` and `{:error, reason}` tuples for functions that can fail.\n  - Use exceptions primarily for programmer errors (e.g., invalid arguments).\n  - Document expected error returns in the function's documentation.\n\n#### Test Requirements\n\n- All new code must have comprehensive tests using [ExUnit](https://hexdocs.pm/ex_unit/ExUnit.html).\n- Pull requests must maintain or improve the current test coverage. We aim for >95% coverage.\n- Tests should cover:\n  - Basic functionality (the \"happy path\").\n  - Edge cases (e.g., empty lists, zero values, nils).\n  - Error handling (invalid inputs, expected failures).\n\nExample test structure:\n\n```\ndefmodule Eliot.MyModuleTest do\n  use ExUnit.Case, async: true\n\n  describe \"my_function/1\" do\n    test \"returns :ok for valid input\" do\n      # Given: valid input\n      input = \"valid\"\n\n      # When: the function is called\n      result = Eliot.MyModule.my_function(input)\n\n      # Then: the result is as expected\n      assert {:ok, \"processed\"} == result\n    end\n\n    test \"returns :error for invalid input\" do\n      assert {:error, :invalid_input} == Eliot.MyModule.my_function(nil)\n    end\n  end\nend\n```","ref":"contributing.html#code-contributions"},{"type":"extras","title":"Development Setup - Contributing to Eliot","doc":"","ref":"contributing.html#development-setup"},{"type":"extras","title":"Prerequisites - Contributing to Eliot","doc":"- Elixir 1.14+\n- Erlang/OTP 25+","ref":"contributing.html#prerequisites"},{"type":"extras","title":"Building for Development - Contributing to Eliot","doc":"1. Clone your fork:\n```\n   git clone [https://github.com/yourusername/eliot.git](https://github.com/yourusername/eliot.git)\n   cd eliot\n```\n2. Install dependencies:\n```\n   mix deps.get\n```\n3. Run the tests to ensure everything is working:\n```\n   mix test\n```\n4. To run the application in an interactive shell:\n```\n   iex -S mix\n```","ref":"contributing.html#building-for-development"},{"type":"extras","title":"Community Priorities - Contributing to Eliot","doc":"As a project focused on IoT data ingestion, contributions that enhance these aspects are particularly welcome:\n\n1. **Reliability improvements** in the `ErrorHandler` or supervision tree.\n2. **Performance enhancements** for high-throughput message processing.\n3. **Additional protocol support** or improved MQTT feature handling.\n4. **Documentation and examples** that help people integrate Eliot into their systems.\n5. **Security features** that protect against common IoT vulnerabilities.","ref":"contributing.html#community-priorities"},{"type":"extras","title":"Additional Notes - Contributing to Eliot","doc":"","ref":"contributing.html#additional-notes"},{"type":"extras","title":"Git Workflow Tips - Contributing to Eliot","doc":"- Keep your commits atomic and focused on a single issue.\n- Write meaningful commit messages explaining the \"what\" and \"why\" of your changes.\n- Rebase your branch onto the latest `main` before submitting a PR to maintain a clean history.","ref":"contributing.html#git-workflow-tips"},{"type":"extras","title":"Questions? - Contributing to Eliot","doc":"If you have any questions about contributing, please open an issue and tag it with `question`.\n\nThank you for contributing to Eliot and helping create a more robust open-source IoT ecosystem!","ref":"contributing.html#questions"},{"type":"extras","title":"🧪 Eliot Testing Guide 🧪","doc":"# 🧪 Eliot Testing Guide 🧪\n\nWelcome to the Eliot testing documentation! This guide explains how to run tests, understand the test suite, and create new tests for the Eliot system.\n\n## 📋 Table of Contents\n\n- [Test Suite Overview](#test-suite-overview)\n- [Running Tests](#running-tests)\n- [Understanding Test Types](#understanding-test-types)\n- [Test Framework (ExUnit)](#test-framework-exunit)\n- [Writing New Tests](#writing-new-tests)\n- [Test Coverage](#test-coverage)\n- [Continuous Integration](#continuous-integration)\n- [Testing Best Practices](#testing-best-practices)","ref":"testing.html"},{"type":"extras","title":"Test Suite Overview - 🧪 Eliot Testing Guide 🧪","doc":"Eliot includes a comprehensive test suite that validates:\n\n- **Core Functionality**: Correctness of module logic and public APIs.\n- **OTP Behavior**: Proper GenServer/Supervisor initialization, state changes, and shutdown.\n- **Error Handling**: Circuit breaker and retry logic correctness.\n- **Integration**: How different components of the system interact.\n\nTests are organized into two main categories:\n- **Unit Tests**: Testing individual modules and functions in isolation.\n- **Integration Tests**: Testing the interaction between different components (e.g., `ErrorHandler` and `Logger`).","ref":"testing.html#test-suite-overview"},{"type":"extras","title":"Running Tests - 🧪 Eliot Testing Guide 🧪","doc":"","ref":"testing.html#running-tests"},{"type":"extras","title":"Running All Tests - 🧪 Eliot Testing Guide 🧪","doc":"The simplest way to run all tests is using the standard `mix` command:\n\n```\nmix test\n```\n\nThis will compile the project if needed and run all tests found in the `test/` directory.","ref":"testing.html#running-all-tests"},{"type":"extras","title":"Running a Single Test File - 🧪 Eliot Testing Guide 🧪","doc":"To run tests from a single file:\n\n```\nmix test test/eliot/error_handler_test.exs\n```","ref":"testing.html#running-a-single-test-file"},{"type":"extras","title":"Running a Single Test - 🧪 Eliot Testing Guide 🧪","doc":"To run a specific test case from a file, you can specify the line number:\n\n```\nmix test test/eliot/error_handler_test.exs:25\n```","ref":"testing.html#running-a-single-test"},{"type":"extras","title":"Excluding or Including Tests with Tags - 🧪 Eliot Testing Guide 🧪","doc":"You can tag tests to run specific categories. For example, integration tests are tagged with `:integration`.\n\n```\n# Run only integration tests\nmix test --only integration\n\n# Run all tests except integration tests\nmix test --exclude integration\n```","ref":"testing.html#excluding-or-including-tests-with-tags"},{"type":"extras","title":"Understanding Test Types - 🧪 Eliot Testing Guide 🧪","doc":"","ref":"testing.html#understanding-test-types"},{"type":"extras","title":"Unit Tests - 🧪 Eliot Testing Guide 🧪","doc":"Unit tests check individual modules and functions in isolation. These tests focus on:\n- Public function return values for given inputs.\n- GenServer callbacks and state transitions.\n- Logic inside private functions (tested via their public-facing callers).\n\nSample unit test files:\n- `test/eliot/elixir_test.exs`\n- `test/eliot/error_handler_test.exs`\n- `test/eliot/logger_test.exs`","ref":"testing.html#unit-tests"},{"type":"extras","title":"Integration Tests - 🧪 Eliot Testing Guide 🧪","doc":"Integration tests verify that different components work together correctly. For Eliot, this includes:\n- Ensuring an error handled by `ErrorHandler` is correctly logged by `Eliot.Logger`.\n- Verifying the `Eliot.Application` correctly starts and supervises its children.\n\nSample integration test files:\n- `test/eliot/integration/mqtt_integration_test.exs`","ref":"testing.html#integration-tests"},{"type":"extras","title":"Test Framework (ExUnit) - 🧪 Eliot Testing Guide 🧪","doc":"Eliot uses Elixir's built-in testing framework, ExUnit.","ref":"testing.html#test-framework-exunit"},{"type":"extras","title":"Test Structure - 🧪 Eliot Testing Guide 🧪","doc":"Tests are defined inside modules using `use ExUnit.Case`. The `describe` block groups related tests, and each `test` block defines a single test case.\n\n```\ndefmodule Eliot.MyModuleTest do\n  use ExUnit.Case, async: true\n\n  describe \"my_function/2\" do\n    test \"does something correctly\" do\n      assert Eliot.MyModule.my_function(:a, :b) == :ok\n    end\n  end\nend\n```","ref":"testing.html#test-structure"},{"type":"extras","title":"Setup - 🧪 Eliot Testing Guide 🧪","doc":"You can use `setup` or `setup_all` blocks to run setup code before tests. `setup` runs before each test in a block, while `setup_all` runs once for the entire block.\n\n```\ndefmodule Eliot.MyModuleTest do\n  use ExUnit.Case, async: true\n\n  setup do\n    # This runs before each test\n    {:ok, pid} = MyGenServer.start_link()\n    # The returned value is passed to the test context\n    %{pid: pid}\n  end\n\n  test \"sends a message to the genserver\", context do\n    assert :ok == MyGenServer.do_something(context.pid)\n  end\nend\n```","ref":"testing.html#setup"},{"type":"extras","title":"Assertions - 🧪 Eliot Testing Guide 🧪","doc":"ExUnit provides various assertion macros:\n- `assert some_condition == true`\n- `refute some_condition == true`\n- `assert_raise(MyError, fn -> ... end)`\n- `assert_receive(:my_message, 100)`\n- `assert_received(:my_message)`","ref":"testing.html#assertions"},{"type":"extras","title":"Mocking - 🧪 Eliot Testing Guide 🧪","doc":"For isolating components from their dependencies, we recommend using a library like [Mox](https://hex.pm/packages/mox). Mox allows you to define mock implementations of your modules' behaviors for testing.","ref":"testing.html#mocking"},{"type":"extras","title":"Writing New Tests - 🧪 Eliot Testing Guide 🧪","doc":"To create a new unit test:\n\n1. Create a new file in the `test/eliot/` directory, ending with `_test.exs`.\n2. Follow this basic structure:\n```\ndefmodule Eliot.NewFeatureTest do\n  use ExUnit.Case, async: true\n\n  describe \"new_feature_function/1\" do\n    test \"handles valid input correctly\" do\n      # Given valid input\n      input = \"valid\"\n      \n      # When the function is called\n      result = Eliot.NewFeature.new_feature_function(input)\n      \n      # Then the result is correct\n      assert result == :ok\n    end\n\n    test \"handles invalid input with an error tuple\" do\n      assert Eliot.NewFeature.new_feature_function(nil) == {:error, :invalid_input}\n    end\n  end\nend\n```\n3. Run `mix test` to ensure your new tests are picked up and pass.","ref":"testing.html#writing-new-tests"},{"type":"extras","title":"Test Coverage - 🧪 Eliot Testing Guide 🧪","doc":"We aim for high code coverage across the codebase.","ref":"testing.html#test-coverage"},{"type":"extras","title":"Measuring Coverage - 🧪 Eliot Testing Guide 🧪","doc":"You can generate a test coverage report by running:\n```\nmix test --cover\n```\nThis generates a report in `cover/index.html` which you can open in your browser to see which lines of code are covered by tests.","ref":"testing.html#measuring-coverage"},{"type":"extras","title":"Coverage Requirements - 🧪 Eliot Testing Guide 🧪","doc":"- Core functionality in `Eliot`, `Eliot.ErrorHandler`, and `Eliot.Logger` should have >95% coverage.\n- All public API functions must have tests.\n- Error handling paths must be tested explicitly.","ref":"testing.html#coverage-requirements"},{"type":"extras","title":"Continuous Integration - 🧪 Eliot Testing Guide 🧪","doc":"Eliot uses GitHub Actions for continuous integration. The workflow runs automatically on pull requests and includes:\n- Running `mix test`\n- Checking formatting with `mix format --check-formatted`\n- Running static analysis with `mix credo`","ref":"testing.html#continuous-integration"},{"type":"extras","title":"Testing Best Practices - 🧪 Eliot Testing Guide 🧪","doc":"1. **Test One Thing at a Time**: Each test should verify a single, specific behavior.\n2. **Use Descriptive Names**: Test names should clearly describe what is being tested.\n3. **Isolate Tests**: Use `async: true` where possible to run tests concurrently and ensure they don't depend on shared state.\n4. **Test the Public API**: Focus tests on the public-facing functions of your modules, not the private implementation details.\n5. **Test Failure Cases**: Verify that your functions fail appropriately with invalid inputs and that your application recovers gracefully from errors.\n\n---\n\n \n   If you have questions about testing, please reach out via GitHub issues.","ref":"testing.html#testing-best-practices"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.","ref":"changelog.html"},{"type":"extras","title":"v0.2.0 - 2025-06-08 - Changelog","doc":"","ref":"changelog.html#v0-2-0-2025-06-08"},{"type":"extras","title":"Changed - Changelog","doc":"* Major refactor of core application logic for improved stability.\n* Overhauled the testing framework for clarity and robustness.","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"* Comprehensive test suite with coverage increased to over 90%.\n* Detailed tests for logger formatting, including metadata and edge cases.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"* Corrected logger implementation to reliably use the custom telemetry handler in all environments.\n* Resolved multiple test failures and race conditions present in the initial version.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.1.0 - 2025-06-06 - Changelog","doc":"","ref":"changelog.html#v0-1-0-2025-06-06"},{"type":"extras","title":"Added - Changelog","doc":"* Initial release of Eliot.\n* Fault-tolerant OTP application structure.\n* `Eliot.Logger` for structured, centralized logging with telemetry integration.\n* `Eliot.ErrorHandler` with retry and circuit-breaker logic for resilient operations.\n* Comprehensive test suite with unit and integration tests.\n* Full ExDoc documentation for the public API.","ref":"changelog.html#added-1"},{"type":"extras","title":"LICENSE","doc":"GNU GENERAL PUBLIC LICENSE\n                       Version 3, 29 June 2007\n\n Copyright (C) 2007 Free Software Foundation, Inc.  \n Everyone is permitted to copy and distribute verbatim copies\n of this license document, but changing it is not allowed.\n\n                            Preamble\n\n  The GNU General Public License is a free, copyleft license for\nsoftware and other kinds of works.\n\n  The licenses for most software and other practical works are designed\nto take away your freedom to share and change the works.  By contrast,\nthe GNU General Public License is intended to guarantee your freedom to\nshare and change all versions of a program--to make sure it remains free\nsoftware for all its users.  We, the Free Software Foundation, use the\nGNU General Public License for most of our software; it applies also to\nany other work released this way by its authors.  You can apply it to\nyour programs, too.\n\n  When we speak of free software, we are referring to freedom, not\nprice.  Our General Public Licenses are designed to make sure that you\nhave the freedom to distribute copies of free software (and charge for\nthem if you wish), that you receive source code or can get it if you\nwant it, that you can change the software or use pieces of it in new\nfree programs, and that you know you can do these things.\n\n  To protect your rights, we need to prevent others from denying you\nthese rights or asking you to surrender the rights.  Therefore, you have\ncertain responsibilities if you distribute copies of the software, or if\nyou modify it: responsibilities to respect the freedom of others.\n\n  For example, if you distribute copies of such a program, whether\ngratis or for a fee, you must pass on to the recipients the same\nfreedoms that you received.  You must make sure that they, too, receive\nor can get the source code.  And you must show them these terms so they\nknow their rights.\n\n  Developers that use the GNU GPL protect your rights with two steps:\n(1) assert copyright on the software, and (2) offer you this License\ngiving you legal permission to copy, distribute and/or modify it.\n\n  For the developers' and authors' protection, the GPL clearly explains\nthat there is no warranty for this free software.  For both users' and\nauthors' sake, the GPL requires that modified versions be marked as\nchanged, so that their problems will not be attributed erroneously to\nauthors of previous versions.\n\n  Some devices are designed to deny users access to install or run\nmodified versions of the software inside them, although the manufacturer\ncan do so.  This is fundamentally incompatible with the aim of\nprotecting users' freedom to change the software.  The systematic\npattern of such abuse occurs in the area of products for individuals to\nuse, which is precisely where it is most unacceptable.  Therefore, we\nhave designed this version of the GPL to prohibit the practice for those\nproducts.  If such problems arise substantially in other domains, we\nstand ready to extend this provision to those domains in future versions\nof the GPL, as needed to protect the freedom of users.\n\n  Finally, every program is threatened constantly by software patents.\nStates should not allow patents to restrict development and use of\nsoftware on general-purpose computers, but in those that do, we wish to\navoid the special danger that patents applied to a free program could\nmake it effectively proprietary.  To prevent this, the GPL assures that\npatents cannot be used to render the program non-free.\n\n  The precise terms and conditions for copying, distribution and\nmodification follow.\n\n                       TERMS AND CONDITIONS\n\n  0. Definitions.\n\n  \"This License\" refers to version 3 of the GNU General Public License.\n\n  \"Copyright\" also means copyright-like laws that apply to other kinds of\nworks, such as semiconductor masks.\n\n  \"The Program\" refers to any copyrightable work licensed under this\nLicense.  Each licensee is addressed as \"you\".  \"Licensees\" and\n\"recipients\" may be individuals or organizations.\n\n  To \"modify\" a work means to copy from or adapt all or part of the work\nin a fashion requiring copyright permission, other than the making of an\nexact copy.  The resulting work is called a \"modified version\" of the\nearlier work or a work \"based on\" the earlier work.\n\n  A \"covered work\" means either the unmodified Program or a work based\non the Program.\n\n  To \"propagate\" a work means to do anything with it that, without\npermission, would make you directly or secondarily liable for\ninfringement under applicable copyright law, except executing it on a\ncomputer or modifying a private copy.  Propagation includes copying,\ndistribution (with or without modification), making available to the\npublic, and in some countries other activities as well.\n\n  To \"convey\" a work means any kind of propagation that enables other\nparties to make or receive copies.  Mere interaction with a user through\na computer network, with no transfer of a copy, is not conveying.\n\n  An interactive user interface displays \"Appropriate Legal Notices\"\nto the extent that it includes a convenient and prominently visible\nfeature that (1) displays an appropriate copyright notice, and (2)\ntells the user that there is no warranty for the work (except to the\nextent that warranties are provided), that licensees may convey the\nwork under this License, and how to view a copy of this License.  If\nthe interface presents a list of user commands or options, such as a\nmenu, a prominent item in the list meets this criterion.\n\n  1. Source Code.\n\n  The \"source code\" for a work means the preferred form of the work\nfor making modifications to it.  \"Object code\" means any non-source\nform of a work.\n\n  A \"Standard Interface\" means an interface that either is an official\nstandard defined by a recognized standards body, or, in the case of\ninterfaces specified for a particular programming language, one that\nis widely used among developers working in that language.\n\n  The \"System Libraries\" of an executable work include anything, other\nthan the work as a whole, that (a) is included in the normal form of\npackaging a Major Component, but which is not part of that Major\nComponent, and (b) serves only to enable use of the work with that\nMajor Component, or to implement a Standard Interface for which an\nimplementation is available to the public in source code form.  A\n\"Major Component\", in this context, means a major essential component\n(kernel, window system, and so on) of the specific operating system\n(if any) on which the executable work runs, or a compiler used to\nproduce the work, or an object code interpreter used to run it.\n\n  The \"Corresponding Source\" for a work in object code form means all\nthe source code needed to generate, install, and (for an executable\nwork) run the object code and to modify the work, including scripts to\ncontrol those activities.  However, it does not include the work's\nSystem Libraries, or general-purpose tools or generally available free\nprograms which are used unmodified in performing those activities but\nwhich are not part of the work.  For example, Corresponding Source\nincludes interface definition files associated with source files for\nthe work, and the source code for shared libraries and dynamically\nlinked subprograms that the work is specifically designed to require,\nsuch as by intimate data communication or control flow between those\nsubprograms and other parts of the work.\n\n  The Corresponding Source need not include anything that users\ncan regenerate automatically from other parts of the Corresponding\nSource.\n\n  The Corresponding Source for a work in source code form is that\nsame work.\n\n  2. Basic Permissions.\n\n  All rights granted under this License are granted for the term of\ncopyright on the Program, and are irrevocable provided the stated\nconditions are met.  This License explicitly affirms your unlimited\npermission to run the unmodified Program.  The output from running a\ncovered work is covered by this License only if the output, given its\ncontent, constitutes a covered work.  This License acknowledges your\nrights of fair use or other equivalent, as provided by copyright law.\n\n  You may make, run and propagate covered works that you do not\nconvey, without conditions so long as your license otherwise remains\nin force.  You may convey covered works to others for the sole purpose\nof having them make modifications exclusively for you, or provide you\nwith facilities for running those works, provided that you comply with\nthe terms of this License in conveying all material for which you do\nnot control copyright.  Those thus making or running the covered works\nfor you must do so exclusively on your behalf, under your direction\nand control, on terms that prohibit them from making any copies of\nyour copyrighted material outside their relationship with you.\n\n  Conveying under any other circumstances is permitted solely under\nthe conditions stated below.  Sublicensing is not allowed; section 10\nmakes it unnecessary.\n\n  3. Protecting Users' Legal Rights From Anti-Circumvention Law.\n\n  No covered work shall be deemed part of an effective technological\nmeasure under any applicable law fulfilling obligations under article\n11 of the WIPO copyright treaty adopted on 20 December 1996, or\nsimilar laws prohibiting or restricting circumvention of such\nmeasures.\n\n  When you convey a covered work, you waive any legal power to forbid\ncircumvention of technological measures to the extent such circumvention\nis effected by exercising rights under this License with respect to\nthe covered work, and you disclaim any intention to limit operation or\nmodification of the work as a means of enforcing, against the work's\nusers, your or third parties' legal rights to forbid circumvention of\ntechnological measures.\n\n  4. Conveying Verbatim Copies.\n\n  You may convey verbatim copies of the Program's source code as you\nreceive it, in any medium, provided that you conspicuously and\nappropriately publish on each copy an appropriate copyright notice;\nkeep intact all notices stating that this License and any\nnon-permissive terms added in accord with section 7 apply to the code;\nkeep intact all notices of the absence of any warranty; and give all\nrecipients a copy of this License along with the Program.\n\n  You may charge any price or no price for each copy that you convey,\nand you may offer support or warranty protection for a fee.\n\n  5. Conveying Modified Source Versions.\n\n  You may convey a work based on the Program, or the modifications to\nproduce it from the Program, in the form of source code under the\nterms of section 4, provided that you also meet all of these conditions:\n\n    a) The work must carry prominent notices stating that you modified\n    it, and giving a relevant date.\n\n    b) The work must carry prominent notices stating that it is\n    released under this License and any conditions added under section\n    7.  This requirement modifies the requirement in section 4 to\n    \"keep intact all notices\".\n\n    c) You must license the entire work, as a whole, under this\n    License to anyone who comes into possession of a copy.  This\n    License will therefore apply, along with any applicable section 7\n    additional terms, to the whole of the work, and all its parts,\n    regardless of how they are packaged.  This License gives no\n    permission to license the work in any other way, but it does not\n    invalidate such permission if you have separately received it.\n\n    d) If the work has interactive user interfaces, each must display\n    Appropriate Legal Notices; however, if the Program has interactive\n    interfaces that do not display Appropriate Legal Notices, your\n    work need not make them do so.\n\n  A compilation of a covered work with other separate and independent\nworks, which are not by their nature extensions of the covered work,\nand which are not combined with it such as to form a larger program,\nin or on a volume of a storage or distribution medium, is called an\n\"aggregate\" if the compilation and its resulting copyright are not\nused to limit the access or legal rights of the compilation's users\nbeyond what the individual works permit.  Inclusion of a covered work\nin an aggregate does not cause this License to apply to the other\nparts of the aggregate.\n\n  6. Conveying Non-Source Forms.\n\n  You may convey a covered work in object code form under the terms\nof sections 4 and 5, provided that you also convey the\nmachine-readable Corresponding Source under the terms of this License,\nin one of these ways:\n\n    a) Convey the object code in, or embodied in, a physical product\n    (including a physical distribution medium), accompanied by the\n    Corresponding Source fixed on a durable physical medium\n    customarily used for software interchange.\n\n    b) Convey the object code in, or embodied in, a physical product\n    (including a physical distribution medium), accompanied by a\n    written offer, valid for at least three years and valid for as\n    long as you offer spare parts or customer support for that product\n    model, to give anyone who possesses the object code either (1) a\n    copy of the Corresponding Source for all the software in the\n    product that is covered by this License, on a durable physical\n    medium customarily used for software interchange, for a price no\n    more than your reasonable cost of physically performing this\n    conveying of source, or (2) access to copy the\n    Corresponding Source from a network server at no charge.\n\n    c) Convey individual copies of the object code with a copy of the\n    written offer to provide the Corresponding Source.  This\n    alternative is allowed only occasionally and noncommercially, and\n    only if you received the object code with such an offer, in accord\n    with subsection 6b.\n\n    d) Convey the object code by offering access from a designated\n    place (gratis or for a charge), and offer equivalent access to the\n    Corresponding Source in the same way through the same place at no\n    further charge.  You need not require recipients to copy the\n    Corresponding Source along with the object code.  If the place to\n    copy the object code is a network server, the Corresponding Source\n    may be on a different server (operated by you or a third party)\n    that supports equivalent copying facilities, provided you maintain\n    clear directions next to the object code saying where to find the\n    Corresponding Source.  Regardless of what server hosts the\n    Corresponding Source, you remain obligated to ensure that it is\n    available for as long as needed to satisfy these requirements.\n\n    e) Convey the object code using peer-to-peer transmission, provided\n    you inform other peers where the object code and Corresponding\n    Source of the work are being offered to the general public at no\n    charge under subsection 6d.\n\n  A separable portion of the object code, whose source code is excluded\nfrom the Corresponding Source as a System Library, need not be\nincluded in conveying the object code work.\n\n  A \"User Product\" is either (1) a \"consumer product\", which means any\ntangible personal property which is normally used for personal, family,\nor household purposes, or (2) anything designed or sold for incorporation\ninto a dwelling.  In determining whether a product is a consumer product,\ndoubtful cases shall be resolved in favor of coverage.  For a particular\nproduct received by a particular user, \"normally used\" refers to a\ntypical or common use of that class of product, regardless of the status\nof the particular user or of the way in which the particular user\nactually uses, or expects or is expected to use, the product.  A product\nis a consumer product regardless of whether the product has substantial\ncommercial, industrial or non-consumer uses, unless such uses represent\nthe only significant mode of use of the product.\n\n  \"Installation Information\" for a User Product means any methods,\nprocedures, authorization keys, or other information required to install\nand execute modified versions of a covered work in that User Product from\na modified version of its Corresponding Source.  The information must\nsuffice to ensure that the continued functioning of the modified object\ncode is in no case prevented or interfered with solely because\nmodification has been made.\n\n  If you convey an object code work under this section in, or with, or\nspecifically for use in, a User Product, and the conveying occurs as\npart of a transaction in which the right of possession and use of the\nUser Product is transferred to the recipient in perpetuity or for a\nfixed term (regardless of how the transaction is characterized), the\nCorresponding Source conveyed under this section must be accompanied\nby the Installation Information.  But this requirement does not apply\nif neither you nor any third party retains the ability to install\nmodified object code on the User Product (for example, the work has\nbeen installed in ROM).\n\n  The requirement to provide Installation Information does not include a\nrequirement to continue to provide support service, warranty, or updates\nfor a work that has been modified or installed by the recipient, or for\nthe User Product in which it has been modified or installed.  Access to a\nnetwork may be denied when the modification itself materially and\nadversely affects the operation of the network or violates the rules and\nprotocols for communication across the network.\n\n  Corresponding Source conveyed, and Installation Information provided,\nin accord with this section must be in a format that is publicly\ndocumented (and with an implementation available to the public in\nsource code form), and must require no special password or key for\nunpacking, reading or copying.\n\n  7. Additional Terms.\n\n  \"Additional permissions\" are terms that supplement the terms of this\nLicense by making exceptions from one or more of its conditions.\nAdditional permissions that are applicable to the entire Program shall\nbe treated as though they were included in this License, to the extent\nthat they are valid under applicable law.  If additional permissions\napply only to part of the Program, that part may be used separately\nunder those permissions, but the entire Program remains governed by\nthis License without regard to the additional permissions.\n\n  When you convey a copy of a covered work, you may at your option\nremove any additional permissions from that copy, or from any part of\nit.  (Additional permissions may be written to require their own\nremoval in certain cases when you modify the work.)  You may place\nadditional permissions on material, added by you to a covered work,\nfor which you have or can give appropriate copyright permission.\n\n  Notwithstanding any other provision of this License, for material you\nadd to a covered work, you may (if authorized by the copyright holders of\nthat material) supplement the terms of this License with terms:\n\n    a) Disclaiming warranty or limiting liability differently from the\n    terms of sections 15 and 16 of this License; or\n\n    b) Requiring preservation of specified reasonable legal notices or\n    author attributions in that material or in the Appropriate Legal\n    Notices displayed by works containing it; or\n\n    c) Prohibiting misrepresentation of the origin of that material, or\n    requiring that modified versions of such material be marked in\n    reasonable ways as different from the original version; or\n\n    d) Limiting the use for publicity purposes of names of licensors or\n    authors of the material; or\n\n    e) Declining to grant rights under trademark law for use of some\n    trade names, trademarks, or service marks; or\n\n    f) Requiring indemnification of licensors and authors of that\n    material by anyone who conveys the material (or modified versions of\n    it) with contractual assumptions of liability to the recipient, for\n    any liability that these contractual assumptions directly impose on\n    those licensors and authors.\n\n  All other non-permissive additional terms are considered \"further\nrestrictions\" within the meaning of section 10.  If the Program as you\nreceived it, or any part of it, contains a notice stating that it is\ngoverned by this License along with a term that is a further\nrestriction, you may remove that term.  If a license document contains\na further restriction but permits relicensing or conveying under this\nLicense, you may add to a covered work material governed by the terms\nof that license document, provided that the further restriction does\nnot survive such relicensing or conveying.\n\n  If you add terms to a covered work in accord with this section, you\nmust place, in the relevant source files, a statement of the\nadditional terms that apply to those files, or a notice indicating\nwhere to find the applicable terms.\n\n  Additional terms, permissive or non-permissive, may be stated in the\nform of a separately written license, or stated as exceptions;\nthe above requirements apply either way.\n\n  8. Termination.\n\n  You may not propagate or modify a covered work except as expressly\nprovided under this License.  Any attempt otherwise to propagate or\nmodify it is void, and will automatically terminate your rights under\nthis License (including any patent licenses granted under the third\nparagraph of section 11).\n\n  However, if you cease all violation of this License, then your\nlicense from a particular copyright holder is reinstated (a)\nprovisionally, unless and until the copyright holder explicitly and\nfinally terminates your license, and (b) permanently, if the copyright\nholder fails to notify you of the violation by some reasonable means\nprior to 60 days after the cessation.\n\n  Moreover, your license from a particular copyright holder is\nreinstated permanently if the copyright holder notifies you of the\nviolation by some reasonable means, this is the first time you have\nreceived notice of violation of this License (for any work) from that\ncopyright holder, and you cure the violation prior to 30 days after\nyour receipt of the notice.\n\n  Termination of your rights under this section does not terminate the\nlicenses of parties who have received copies or rights from you under\nthis License.  If your rights have been terminated and not permanently\nreinstated, you do not qualify to receive new licenses for the same\nmaterial under section 10.\n\n  9. Acceptance Not Required for Having Copies.\n\n  You are not required to accept this License in order to receive or\nrun a copy of the Program.  Ancillary propagation of a covered work\noccurring solely as a consequence of using peer-to-peer transmission\nto receive a copy likewise does not require acceptance.  However,\nnothing other than this License grants you permission to propagate or\nmodify any covered work.  These actions infringe copyright if you do\nnot accept this License.  Therefore, by modifying or propagating a\ncovered work, you indicate your acceptance of this License to do so.\n\n  10. Automatic Licensing of Downstream Recipients.\n\n  Each time you convey a covered work, the recipient automatically\nreceives a license from the original licensors, to run, modify and\npropagate that work, subject to this License.  You are not responsible\nfor enforcing compliance by third parties with this License.\n\n  An \"entity transaction\" is a transaction transferring control of an\norganization, or substantially all assets of one, or subdividing an\norganization, or merging organizations.  If propagation of a covered\nwork results from an entity transaction, each party to that\ntransaction who receives a copy of the work also receives whatever\nlicenses to the work the party's predecessor in interest had or could\ngive under the previous paragraph, plus a right to possession of the\nCorresponding Source of the work from the predecessor in interest, if\nthe predecessor has it or can get it with reasonable efforts.\n\n  You may not impose any further restrictions on the exercise of the\nrights granted or affirmed under this License.  For example, you may\nnot impose a license fee, royalty, or other charge for exercise of\nrights granted under this License, and you may not initiate litigation\n(including a cross-claim or counterclaim in a lawsuit) alleging that\nany patent claim is infringed by making, using, selling, offering for\nsale, or importing the Program or any portion of it.\n\n  11. Patents.\n\n  A \"contributor\" is a copyright holder who authorizes use under this\nLicense of the Program or a work on which the Program is based.  The\nwork thus licensed is called the contributor's \"contributor version\".\n\n  A contributor's \"essential patent claims\" are all patent claims\nowned or controlled by the contributor, whether already acquired or\nhereafter acquired, that would be infringed by some manner, permitted\nby this License, of making, using, or selling its contributor version,\nbut do not include claims that would be infringed only as a\nconsequence of further modification of the contributor version.  For\npurposes of this definition, \"control\" includes the right to grant\npatent sublicenses in a manner consistent with the requirements of\nthis License.\n\n  Each contributor grants you a non-exclusive, worldwide, royalty-free\npatent license under the contributor's essential patent claims, to\nmake, use, sell, offer for sale, import and otherwise run, modify and\npropagate the contents of its contributor version.\n\n  In the following three paragraphs, a \"patent license\" is any express\nagreement or commitment, however denominated, not to enforce a patent\n(such as an express permission to practice a patent or covenant not to\nsue for patent infringement).  To \"grant\" such a patent license to a\nparty means to make such an agreement or commitment not to enforce a\npatent against the party.\n\n  If you convey a covered work, knowingly relying on a patent license,\nand the Corresponding Source of the work is not available for anyone\nto copy, free of charge and under the terms of this License, through a\npublicly available network server or other readily accessible means,\nthen you must either (1) cause the Corresponding Source to be so\navailable, or (2) arrange to deprive yourself of the benefit of the\npatent license for this particular work, or (3) arrange, in a manner\nconsistent with the requirements of this License, to extend the patent\nlicense to downstream recipients.  \"Knowingly relying\" means you have\nactual knowledge that, but for the patent license, your conveying the\ncovered work in a country, or your recipient's use of the covered work\nin a country, would infringe one or more identifiable patents in that\ncountry that you have reason to believe are valid.\n\n  If, pursuant to or in connection with a single transaction or\narrangement, you convey, or propagate by procuring conveyance of, a\ncovered work, and grant a patent license to some of the parties\nreceiving the covered work authorizing them to use, propagate, modify\nor convey a specific copy of the covered work, then the patent license\nyou grant is automatically extended to all recipients of the covered\nwork and works based on it.\n\n  A patent license is \"discriminatory\" if it does not include within\nthe scope of its coverage, prohibits the exercise of, or is\nconditioned on the non-exercise of one or more of the rights that are\nspecifically granted under this License.  You may not convey a covered\nwork if you are a party to an arrangement with a third party that is\nin the business of distributing software, under which you make payment\nto the third party based on the extent of your activity of conveying\nthe work, and under which the third party grants, to any of the\nparties who would receive the covered work from you, a discriminatory\npatent license (a) in connection with copies of the covered work\nconveyed by you (or copies made from those copies), or (b) primarily\nfor and in connection with specific products or compilations that\ncontain the covered work, unless you entered into that arrangement,\nor that patent license was granted, prior to 28 March 2007.\n\n  Nothing in this License shall be construed as excluding or limiting\nany implied license or other defenses to infringement that may\notherwise be available to you under applicable patent law.\n\n  12. No Surrender of Others' Freedom.\n\n  If conditions are imposed on you (whether by court order, agreement or\notherwise) that contradict the conditions of this License, they do not\nexcuse you from the conditions of this License.  If you cannot convey a\ncovered work so as to satisfy simultaneously your obligations under this\nLicense and any other pertinent obligations, then as a consequence you may\nnot convey it at all.  For example, if you agree to terms that obligate you\nto collect a royalty for further conveying from those to whom you convey\nthe Program, the only way you could satisfy both those terms and this\nLicense would be to refrain entirely from conveying the Program.\n\n  13. Use with the GNU Affero General Public License.\n\n  Notwithstanding any other provision of this License, you have\npermission to link or combine any covered work with a work licensed\nunder version 3 of the GNU Affero General Public License into a single\ncombined work, and to convey the resulting work.  The terms of this\nLicense will continue to apply to the part which is the covered work,\nbut the special requirements of the GNU Affero General Public License,\nsection 13, concerning interaction through a network will apply to the\ncombination as such.\n\n  14. Revised Versions of this License.\n\n  The Free Software Foundation may publish revised and/or new versions of\nthe GNU General Public License from time to time.  Such new versions will\nbe similar in spirit to the present version, but may differ in detail to\naddress new problems or concerns.\n\n  Each version is given a distinguishing version number.  If the\nProgram specifies that a certain numbered version of the GNU General\nPublic License \"or any later version\" applies to it, you have the\noption of following the terms and conditions either of that numbered\nversion or of any later version published by the Free Software\nFoundation.  If the Program does not specify a version number of the\nGNU General Public License, you may choose any version ever published\nby the Free Software Foundation.\n\n  If the Program specifies that a proxy can decide which future\nversions of the GNU General Public License can be used, that proxy's\npublic statement of acceptance of a version permanently authorizes you\nto choose that version for the Program.\n\n  Later license versions may give you additional or different\npermissions.  However, no additional obligations are imposed on any\nauthor or copyright holder as a result of your choosing to follow a\nlater version.\n\n  15. Disclaimer of Warranty.\n\n  THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY\nAPPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT\nHOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY\nOF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,\nTHE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR\nPURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM\nIS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF\nALL NECESSARY SERVICING, REPAIR OR CORRECTION.\n\n  16. Limitation of Liability.\n\n  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING\nWILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS\nTHE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY\nGENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE\nUSE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF\nDATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD\nPARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),\nEVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF\nSUCH DAMAGES.\n\n  17. Interpretation of Sections 15 and 16.\n\n  If the disclaimer of warranty and limitation of liability provided\nabove cannot be given local legal effect according to their terms,\nreviewing courts shall apply local law that most closely approximates\nan absolute waiver of all civil liability in connection with the\nProgram, unless a warranty or assumption of liability accompanies a\ncopy of the Program in return for a fee.\n\n                     END OF TERMS AND CONDITIONS\n\n            How to Apply These Terms to Your New Programs\n\n  If you develop a new program, and you want it to be of the greatest\npossible use to the public, the best way to achieve this is to make it\nfree software which everyone can redistribute and change under these terms.\n\n  To do so, attach the following notices to the program.  It is safest\nto attach them to the start of each source file to most effectively\nstate the exclusion of warranty; and each file should have at least\nthe \"copyright\" line and a pointer to where the full notice is found.\n\n     \n    Copyright (C)     \n\n    This program is free software: you can redistribute it and/or modify\n    it under the terms of the GNU General Public License as published by\n    the Free Software Foundation, either version 3 of the License, or\n    (at your option) any later version.\n\n    This program is distributed in the hope that it will be useful,\n    but WITHOUT ANY WARRANTY; without even the implied warranty of\n    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n    GNU General Public License for more details.\n\n    You should have received a copy of the GNU General Public License\n    along with this program.  If not, see  .\n\nAlso add information on how to contact you by electronic and paper mail.\n\n  If the program does terminal interaction, make it output a short\nnotice like this when it starts in an interactive mode:\n\n       Copyright (C)     \n    This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.\n    This is free software, and you are welcome to redistribute it\n    under certain conditions; type `show c' for details.\n\nThe hypothetical commands `show w' and `show c' should show the appropriate\nparts of the General Public License.  Of course, your program's commands\nmight be different; for a GUI interface, you would use an \"about box\".\n\n  You should also get your employer (if you work as a programmer) or school,\nif any, to sign a \"copyright disclaimer\" for the program, if necessary.\nFor more information on this, and how to apply and follow the GNU GPL, see\n .\n\n  The GNU General Public License does not permit incorporating your program\ninto proprietary programs.  If your program is a subroutine library, you\nmay consider it more useful to permit linking proprietary applications with\nthe library.  If this is what you want to do, use the GNU Lesser General\nPublic License instead of this License.  But first, please read\n .","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}